#!/bin/bash -x
##############################################################################
# Copyright (c) Melkon Company. 2011-2012
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
##############################################################################
#
# NAME :        scms-emi-install.sh
#
# DESCRIPTION : SCMS-EMI automatic installation script on the basis 
#		of detailed instructions for manual installation
#		http://scms.pro/install.html
#
# AUTHORS :     L.Belous (belous <at> ilt.kharkov.ua)
#
# NOTES :	for GNU Linux redhat el5(6) distributions (CentOS, SL)
#		version of the combined servers 
#		( frontend + webserver on the single host )
#
# DATA :        02.08.2012
#
##############################################################################

export HOME_INST=`pwd`;

###@ STEP 0 - downloading SCMS-EMI distributive from scms.pro
wget http://scms.pro/files/releases/scms-emi-latest.tar.gz
tar xzf scms-emi-latest.tar.gz
chown -R root.root scms-emi
mv scms-emi/* .
rmdir scms-emi
 
###@ STEP 1 - check architecture, os, release
# m - x86 or x86_64

arch=`uname -m`
if [ $arch = "x86_64" ]  
then 
        echo "OK - arch = x86_64"
else
        echo "ERROR:  The script is not jet support this architecture $arch "
        exit 1
fi 

# os - CentOS or Scientific
os=`cat /etc/*-rel*|sed q|awk -F' ' '{print $1}'`

# elx - release OS = 5 or 6
elx=`cat /etc/*-rel*|sed q|sed -e 's/\(.*\) \(.*\) \(.*\)/\2/'|awk -F'.' '{print $1}'`

if [[ (( $elx = "5" ) || ( $elx = "6" ))&&(( $os = "Scientific" ) || ($os = "CentOS" ))]]
then
      echo "OK - os = $os , release = $elx"
else
      echo "ERROR: The script is not jet support this release `cat /etc/*-rel*|sed q`"
      exit 2
fi

###@ STEP 2 - adding to PATH, init $mysql_passwd, $ip, %hostname
export PATH=/sbin:/usr/sbin:$PATH
export mysql_passwd=`cat config/mysql_passwd`
export hostname=`hostname -f`

# bind-utils for host command
yum -y install bind-utils
export ip=`/usr/bin/host $hostname | awk -F' ' '{printf $4}'`

###@ STEP 3 - upgrade OS with mandatory componens 
yum -y install mod_ssl

if [ $elx = "5" ]; then
   # in old distributive PHP-5.1 with problem JSON undefined function
   yum -y install php53-common
   yum -y install php53
   yum -y install mysql
   yum -y install mysql-server
   yum -y install php53-mysql
   yum -y install php53-mbstring
   yum -y install php53-ldap        
else
   # el6
   yum -y install mysql
   yum -y install mysql-server
   yum -y install php
   yum -y install php-mysql
   yum -y install php-common php-mbstring php-mcrypt php-devel php-xml php-gd
fi

yum -y install perl-Config-General.noarch
yum -y install perl-TimeDate.noarch
yum -y install perl-Expect
yum -y install perl-IO-Stty

if [ $os = "CentOS" ] && [ $elx = "5" ]; then
        yum -y --nogpgcheck install perl-JSON
else
        yum -y install perl-JSON
fi

yum -y install perl-LDAP.noarch
yum -y install php-pear
yum -y install rsync
yum -y install cronie
yum -y install gcc
yum -y install make
yum -y install pam-devel
pecl install PAM

###@ STEP 4 - install standalone nordugrid client ($elx = 5 or 6) in /opt/
wget http://download.nordugrid.org/software/11.05-2/redhat/el$elx/x86_64/nordugrid-arc-standalone-11.05-2.el$elx.x86_64.tgz -P /opt/

/bin/tar xpzf /opt/nordugrid-arc-standalone-* -C /opt/
rm -f /opt/*tgz
/bin/ln -s /opt/nordugrid* /opt/nordugrid


###@ STEP 5 - install hydra

# define new privatekey.h with changed HANDSHAKE1,2 keys
#   as changed number with 10 digits we use command 'date +%s'
x1=`date +%s`;
x2=`echo $x1|awk '{print substr($1, 1, 9)}'`;
h1="\"632465789${x2}\"";
h2="\"${x2}987654321\"";
echo "// Change this keys before compiling for cluster" > /tmp/r;
echo "#define HANDSHAKE1        $h1" >> /tmp/r;
echo "#define HANDSHAKE1LEN    (sizeof(HANDSHAKE1)-1)" >> /tmp/r;
echo "#define HANDSHAKE2        $h2" >> /tmp/r;
echo "#define HANDSHAKE2LEN    (sizeof(HANDSHAKE2)-1)" >> /tmp/r;
mv /tmp/r $HOME_INST/hydra/src/privatekey.h;

# configure, make, make istall hydra
# before configure hydra we need see the nordugrid commands
cd /opt/nordugrid; 
source ./setup.sh;

cd $HOME_INST/hydra
./configure --enable-server --enable-client --prefix=/opt/hydra --enable-arc
make
make install

# configuration for arc-standalone x86_64 
cp  $HOME_INST/config/grid.conf > /opt/hydra/etc/

chgrp -R apache /opt/hydra
cp /opt/hydra/etc/init.d/hydrad /etc/init.d/
chkconfig hydrad --level 3 on
service hydrad start

###@ STEP 6 - prepair mysql service
/sbin/service mysqld start
chkconfig mysqld on
  
# install root passwd for mysql
/usr/bin/mysqladmin -u root password $mysql_passwd
/usr/bin/mysqladmin -u root -h $hostname password $mysql_passwd

###@ STEP 7 - Changes in /etc/php.ini
# delete lines which will be changed
cp /etc/php.ini /tmp/php.ini-0
cat /etc/php.ini | sed '/safe_mode/d' 		> /tmp/r0
cat /tmp/r0      | sed '/short_open_tag/d' 	> /tmp/r1
cat /tmp/r1      | sed '/display_errors/d' 	> /tmp/r0
cat /tmp/r0      | sed '/register_globals/d' 	> /tmp/r1
cat /tmp/r1      | sed '/magic_quotes_gpc/d' 	> /tmp/r0
cat /tmp/r0      | sed '/post_max_size/d' 	> /tmp/r1
cat /tmp/r1      | sed '/upload_max_filesize/d'	> /tmp/r0
cat /tmp/r0      | sed '/default_socket_timeout/d' > /etc/php.ini
cat $HOME_INST/config/to_php_ini >> /etc/php.ini

###@ STEP 8 - create and assign initial state of data base "scms-emi"
cat $HOME_INST/config/scms-emi-create.sql | sed "s/mysql-pass/$mysql_passwd/g" > /tmp/r1;
mv /tmp/r1 $HOME_INST/config/scms-emi-create.sql;  
mysql -uroot -p$mysql_passwd < $HOME_INST/config/scms-emi-create.sql;
 

###@ STEP 9 - install /var/www/scms-emi from distributive
cp -R $HOME_INST/webapp  /var/www/
export HOME_SCMS_EMI=/var/www/scms-emi
mv /var/www/webapp $HOME_SCMS_EMI
chown -R apache.apache $HOME_SCMS_EMI


###@ STEP 10 - set right "r" for shadow for apache (it is needed for pam_auth)
chgrp apache /etc/shadow
chmod 040 /etc/shadow

###@ STEP 11 - tune pam with preinstalled configuration 
mv /etc/pam.d /etc/pam.d-old
/bin/tar xpzf  $HOME_INST/config/pam.d.tgz -C /etc

###@ STEP 12 - tune httpd/conf.d
cp -R $HOME_INST/config/scms-emi.conf /etc/httpd/conf.d/
mv /var/www/html /var/www/html-old
cd /var/www
ln -s scms-emi html

###@ STEP 13 - install crontab for root
/usr/bin/crontab $HOME_INST/config/f_crontab

first=`egrep '###@' /etc/rc.local`

if [ "$first" = "" ]; then
	
	###@ STEP 14 - add to /etc/rc.local 
	cp /etc/rc.local /etc/rc.local-
	echo " " >> /etc/rc.local
	echo "###@ initiate NorduGrid environment" >> /etc/rc.local
	echo "oldpath=`pwd`" >> /etc/rc.local
	echo "cd /opt/nordugrid; source setup.sh; cd \$oldpath" >> /etc/rc.local
	echo " " >> /etc/rc.local
	echo "###@ start hydrad daemon" >> /etc/rc.local
	echo "service hydrad restart" >> /etc/rc.local

	###@ STEP 15 - correct /etc/httpd/conf/*.conf
	# obsolete correction
	# rm -f /var/tmp/*
	
	###@ STEP 16 - switch on httpd
	chkconfig httpd on 
	chkconfig hydrad on 
	 
	###@ STEP 17 - renew httpd after correction config
	service httpd restart 
	
	###@ STEP 18 - correct mysql_passwd, path to hctl and ip in config.php
	cphp=/var/www/scms-emi/site/config.php
	cp $cphp ${cphp}-old
	sed -e "s/mysql_pass, ''/mysql_pass, '$mysql_passwd'/g" $cphp > /var/tmp/cphp1
	sed -e "s-'/path/to/hydra/bin/hctl'-'/opt/hydra/bin/hctl'-g" /var/tmp/cphp1 > /var/tmp/cphp2
	sed -e "s/ldap/pam/g" /var/tmp/cphp2 > /var/tmp/cphp3
	sed -e "s-/path/to/hydra/bin/hctl-/opt/hydra/bin/hctl-g" /var/tmp/cphp3 > /var/tmp/cphp4 
	sed -e "s/_hydrad_host, ''/_hydrad_host, \'$ip\'/g" /var/tmp/cphp4 > $cphp 
	
	###@ STEP 19 - correct /opt/hydra/etc/mysql.conf 
	cphp=/opt/hydra/etc/mysql.conf
	cp $cphp ${cphp}-old
	sed -e "s/MYSQL_PASSWD=xxxxx/MYSQL_PASSWD=$mysql_passwd/g" $cphp > /var/tmp/config1
	sed -e "s/MYSQL_SERVER=webserver/MYSQL_SERVER=$hostname/g" /var/tmp/config1 > /var/tmp/config2
	sed -e "s/MYSQL_DATABASE=scms/MYSQL_DATABASE=scms-emi/g" /var/tmp/config2 > $cphp
#	rm -f /var/tmp/*

	###@ STEP 20 - correct /opt/hydra/etc/grid.conf for case standalone nordugrid only!  
        cgrid=/opt/hydra/etc/grid.conf
        cp $cgrid ${cgrid}-old
        sed -e "s-/usr/bin-/opt/nordugrid/bin-g" $cgrid > /var/tmp/cgrid
	cp /var/tmp/cgrid $cgrid
#       rm -f /var/tmp/*

else
	echo "ERROR: instal-scms - 2-nd start of script! You should start from begin!"; 

fi

###@ STEP 21 - correct timezone
cp /usr/share/zoneinfo/Europe/Kiev /etc/localtime

###@ STEP 22 - load initial state of emi-scms database	
cd $HOME_SCMS_EMI/install && php -c /etc/php.ini ./install.php

###@ STEP 23 - move install dir (instead delete by instruction)
mv $HOME_SCMS_EMI/install $HOME_SCMS_EMI/install-

###@ STEP 24 - setup environment for arc standalone
sed '2 i\
cd /opt/nordugrid' /opt/nordugrid/setup.sh > /etc/profile.d/nordugrid-env-setup.sh

echo " "
echo "********* INSTALLATION OF SCMS ENDED *******************" 
